<?php


namespace App\Repositories;


use App\Models\CardRidingOrder;
use Carbon\Carbon;

class CardRidingOrderRepository extends BaseRepository
{
    public function __construct(CardRidingOrder $cardRidingOrder)
    {
        $this->model = $cardRidingOrder;
    }

    /**
     * 判断用户是否有已支付 但是未回调成功的记录
     *
     * @param $user_id integer
     *
     * @return bool 为true时通过   为false失败
     *
     * */
    public function isPayCallbackOk($user_id)
    {
        $cardRidingOrder = $this->model->query()
            ->where('user_id', $user_id)
            ->where('pay_status', CardRidingOrder::PAY_STATUS_NO)
            ->where('status', CardRidingOrder::STATUS_OK)
            ->first();
        if (empty($cardRidingOrder)) {
            // 验证通过
            return true;
        }
        // 验证失败 存在订单 已支付但未回调
        return false;
    }


    /**
     * 根据用户id 获取历史订单 半年内
     *
     * */
    public function getAllByUserId($user_id)
    {
        $time = Carbon::now()->subMonths(6);
        $cardRidingOrder = $this->model->query()
            ->where('user_id', $user_id)
            ->where('created_at', '>', $time)
            ->where('pay_status', CardRidingOrder::PAY_STATUS_OK)
            ->orderByDesc('id')
            ->paginate();

        return $cardRidingOrder;
    }
}
