<?php

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/

$api = app('Dingo\Api\Routing\Router');

/** @var TYPE_NAME $api */
$api->version('v1', [
    'namespace' => 'App\Http\Controllers\V1',
    'middleware' => ['serializer:array', 'cors']// 'bindings'
], function (\Dingo\Api\Routing\Router $api) {
    $api->get('/t', 'ExampleController@test');

    $api->get('/d', 'ExampleController@decode');
    $api->get('/check', 'ExampleController@check');

    $api->get('/user/token', 'ExampleController@token');

    $api->get('/test/order-refund', 'ExampleController@orderRefund');

    // 广告管理
    $api->get('/ads', 'AdvertisementController@index');
    $api->get('/ad', 'AdvertisementController@show');

    // 公告
    $api->get('/ans', 'AnnouncementController@index');
    $api->get('/an', 'AnnouncementController@show');
    $api->get('/index/an', 'AnnouncementController@indexShow');

    //h5招商
    $api->post('/h5/business', 'H5Controller@business');


//    微信小程序授权登录
    $api->post('/auth/weapp-login', 'AuthController@weappLogin');
    // 认证通过
    $api->group(['middleware' => ['auth:api', 'singleLogin']], function ($api) {
//        微信同步用户信息
        $api->post('/auth/weapp-userinfo-sync', 'AuthController@weappUserInfoSync');
        $api->post('/auth/mobileLogin', 'AuthController@mobileLogin');


        //获取当前登录用户
        $api->get('/me', 'AuthController@loginUser');

        //获取用户的状态
        $api->get('/user/status', "UserController@userStatus");
        $api->get('/user/wallerLogList', "UserController@userWallerLogList");


        // 邀请新用户
        $api->get('/user/getInviteWechatQrcode', 'UserController@getInviteWechatQrcode');// 获取邀请新用户二维码
        $api->get('/user/getInvitePoster', 'UserController@getInvitePoster');        // 获取邀请海报
        $api->post('/user/getInviteNewUsersReward', 'UserController@getInviteNewUsersReward');// 获取奖励
        $api->get('/user/getInviteUsers', 'UserController@getInviteUsers'); // 获取邀请的用户

        // 获取邀请新用户活动配置
        $api->get('/pages/inviteNewusersConfigs', 'PageController@inviteNewusersConfigs');


        /**
         * 页面初始化
         */

//    首页
        $api->post('/pages/home', "PageController@home");

        $api->get('/pages/ride-order', "PageController@rideOrder");

        //押金退款状态
        $api->get('/pages/user-deposit-status', 'PageController@userDepositStatus');


        // 申请增加还车点
        $api->post('/pages/applyAddParking', "PageController@applyAddParking");
        /**
         * 车
         */
//    根据位置找车
        $api->post('/bikes', "BikeController@index");

//        全部区域
        $api->get('/areas', 'AreaController@index');

//    根据位置找车和停车区

//    根据位置查找停车区
        $api->get('/parking/stop-sites', 'ParkingController@stopSite');
//        查找禁停区
        $api->get('/parking/ban-stop-sites', 'ParkingController@banStopSite');

//    单个车的信息
        $api->get('/bike/{bike_no:[0-9]+}', "BikeController@show");

//    响铃寻车
        $api->get('/bike/find/{bike_no:[0-9]+}', "BikeController@findBike");

        // 订单重试开锁
        $api->get('/order/retry-open-lock', "OrderController@retryOpenLock");
        // 订单重试关锁
        $api->get('/order/retry-close-lock', "OrderController@retryCloseLock");
        // 订单重试响铃
        $api->get('/order/retry-bell-lock', "OrderController@retryBellLock");

        //获取当前骑行费用
        $api->post('/order/expect-order-money', 'OrderController@expectOrderMoney');

        //检查是否可以还车
        $api->post('/bike/check-is-riding-area', 'BikeController@checkBikeIsRidingArea');

//        开锁
        $api->post('/bike/open-lock', "BikeController@openLock");
        //锁车
        $api->post('/bike/close-lock', "BikeController@closeLock");

        //是否在停车区
        $api->post('/bike/is-stop-parking', 'BikeController@checkBikeIsInStopParking');

        //临时停车
        $api->post('/bike/temporary-close-lock', "BikeController@temporaryCloseLock");
        //临时开车
        $api->post('/bike/temporary-open-lock', "BikeController@temporaryOpenLock");

        //获取蓝牙秘钥
        $api->get('/blu/get-key', 'BluController@getKey');

        //获取骑行订单
        $api->get('/order/ride-status', 'OrderController@orderRideStatus');
        // 获取订单支付页面接口
        $api->get('/order/pay-show', 'OrderController@payShow');

        // 优惠券
        $api->get('/coupon', 'CouponController@index');
        $api->get('/coupon/getCouponsByOrder', 'CouponController@getCouponsByOrder');


        // 检查用户订单（押金，骑行订单，充值）是否支付
        $api->post('/payment/pay-status', 'PaymentController@checkOrderPayStatus');

        // 订单支付请求
        $api->post('/order/pay', "OrderController@pay");
        // 订单车辆位置
        $api->get('/order/orderBikePosition', "OrderController@orderBikePosition");


//        报修提交
        $api->post('/trouble', 'TroubleController@store');
//        上传图片接口
//        $api->post('/upload/image', 'UploadController@image');

        /**
         * 完善个人信息（注册）
         */
//        实名认证
        $api->post('/user/real-name-authentication', 'UserController@realNameAuthentication');

//        手机号绑定
        $api->post('/user/bind-wechat-mobile', 'UserController@bindWechatMobile');
        $api->post('/user/bind-mobile', 'UserController@bindMobile');

        //学生认证
        $api->post('/user/student', 'UserController@students');

        // 绑定用户的form_id
//        $api->post('/user/store-form-id', 'UserController@storeFormId');

//        验证码
        $api->post('/verification-code', 'UserController@sendVerificationCode');
//        支付押金
        $api->post('/deposit/pay', 'DepositOrderController@store');
//        退押金接口（马上）
        $api->post('/deposit/refund', 'DepositOrderController@refund');
        //退押金（队列）
        $api->post('/deposit/refund-job', 'DepositOrderController@refundJob');
//        获取订单列表

        $api->get('/orders', "OrderController@index");
// 获取订单详情
        $api->get('/order', "OrderController@show");

//        充值接口
        $api->post('/rechange/pay', 'RechangeOrderController@store');

        //问题反馈
        $api->post('/question', 'QuestionController@store');

        /**
         * 日租车
         */

        //日租下单
        $api->post('/rent/store-order', 'RentBikeController@storeOrder');
        //结束日租订单
        $api->post('/rent/close-order', 'RentBikeController@closeOrder');
        //日租开车
        $api->post('/rent/open-bike', 'RentBikeController@openBike');
        //日租关车
        $api->post('/rent/close-bike', 'RentBikeController@closeBike');
        //日租车响铃寻车
        $api->get('/rent/retry-bell', "RentBikeController@retryBell");

        // 日租订单列表
        $api->get('/rent/orders', 'RentBikeController@orders');
        $api->get('/rent/use-order', 'RentBikeController@useOrder');
        $api->get('/rent/order', 'RentBikeController@show');
        $api->get('/rent/pay-show', 'RentBikeController@payShow');
        $api->post('/rent/pay', 'RentBikeController@pay');
        //是否在停车区
        $api->post('/rent/is-stop-parking', 'RentBikeController@checkBikeIsInStopParking');

        //获取当前骑行费用
        $api->post('/rent/expect-rent-order-money', 'RentBikeController@expectRentOrderMoney');

        //检查是否可以还车
        $api->post('/rent/check-is-riding-area', 'RentBikeController@checkBikeIsRidingArea');


        //**********************   骑行卡   ***************************************************************
        //*
        $api->get('/card_riding/index', 'CardRidingController@index');                                   //*
        $api->post('/card_riding/pay', 'CardRidingController@buyCardRiding');                            //*
        $api->post('/card_riding/no_pay', 'CardRidingController@changeCardRidingOrderStatus');           //*
        $api->get('/card_riding/user_card', 'CardRidingController@userCardRiding');                      //*
        $api->get('/card_riding/user_history_orders', 'CardRidingController@CardRidingOrderHistory');    //*
        //*
        //*************************************************************************************************

        //充值活动参数
        $api->get('/pages/recharge', 'PageController@rechargePage');

        //***************************** 押金卡********************************************************
        $api->get('/deposit_card/index', 'DepositCardOrderController@index');
        $api->post('/deposit_card/pay', 'DepositCardOrderController@buyDepositCard');
        //***************************** 罚单***********
        $api->get('/punishment_orders', 'PunishmentOrderController@index');
        $api->get('/punishment_order', 'PunishmentOrderController@show');
        $api->post('/punishment_order/pay', 'PunishmentOrderController@pay');
    });

    // *************************************************************************
    $api->get('/parking/nearby', 'ParkingController@nearbySite');

//    通知公告通知

    // 微信支付接口通知
    $api->any("/payments/wechat-notify", 'PaymentController@notify');
    $api->any("/payments/wechat-rent-notify", 'PaymentController@rentNotify');
    $api->any("/payments/wechat-refund-notify", 'PaymentController@refundNotify');
    $api->any("/payments/wechat-refund-api", 'PaymentController@isOrderRefundPay');

    /**
     * replay端接口
     */
    $api->get('/relay/order/auto-close', 'OrderController@autoCloseOrder');

    /**
     * 对外提供接口
     */
    $api->get('/common/clear-cache', 'ApiController@clearCache');

    $api->post('/upload/image', 'UploadController@image');
});

